@@ -1,25 +1,23 @@
 from __future__ import with_statement
 
 import copy
+from functools import partial
 from operator import isMappingType
-import os
 import sys
-from contextlib import contextmanager
 
-from fudge import Fake, patched_context, with_fakes
+from fudge import Fake, patched_context
 from nose.tools import ok_, eq_
 
 from fabric.decorators import hosts, roles, task
+from fabric.context_managers import settings
 from fabric.main import (parse_arguments, _escape_split,
         load_fabfile as _load_fabfile, list_commands, _task_names,
         COMMANDS_HEADER, NESTED_REMINDER)
 import fabric.state
-from fabric.state import _AttributeDict
 from fabric.tasks import Task, WrappedCallableTask
 from fabric.task_utils import _crawl, crawl, merge
 
-from utils import (mock_streams, patched_env, eq_, FabricTest, fabfile,
-    path_prefix, aborts)
+from utils import FabricTest, fabfile, path_prefix, aborts
 
 
 # Stupid load_fabfile wrapper to hide newly added return value.
@@ -34,7 +32,7 @@ def load_fabfile(*args, **kwargs):
 
 def test_argument_parsing():
     for args, output in [
-        # Basic 
+        # Basic
         ('abc', ('abc', [], {}, [], [], [])),
         # Arg
         ('ab:c', ('ab', ['c'], {}, [], [], [])),
@@ -74,17 +72,37 @@ def test_escaped_task_arg_split():
     )
 
 
+def test_escaped_task_kwarg_split():
+    """
+    Allow backslashes to escape the = in x=y task kwargs
+    """
+    argstr = r"cmd:arg,escaped\,arg,nota\=kwarg,regular=kwarg,escaped=regular\=kwarg"
+    args = ['arg', 'escaped,arg', 'nota=kwarg']
+    kwargs = {'regular': 'kwarg', 'escaped': 'regular=kwarg'}
+    eq_(
+        parse_arguments([argstr])[0],
+        ('cmd', args, kwargs, [], [], []),
+    )
+
+
+
 #
 # Host/role decorators
 #
 
 # Allow calling Task.get_hosts as function instead (meh.)
-def get_hosts(command, *args):
-    print "calling get_hosts with args %r" % (args,)
-    return WrappedCallableTask(command).get_hosts(*args)
+def get_hosts_and_effective_roles(command, *args):
+    return WrappedCallableTask(command).get_hosts_and_effective_roles(*args)
 
-def eq_hosts(command, host_list, env=None):
-    eq_(set(get_hosts(command, [], [], [], env)), set(host_list))
+def eq_hosts(command, expected_hosts, cli_hosts=None, excluded_hosts=None, env=None, func=set):
+    eq_(func(get_hosts_and_effective_roles(command, cli_hosts or [], [], excluded_hosts or [], env)[0]),
+        func(expected_hosts))
+
+def eq_effective_roles(command, expected_effective_roles, cli_roles=None, env=None, func=set):
+    eq_(func(get_hosts_and_effective_roles(command, [], cli_roles or [], [], env)[1]),
+        func(expected_effective_roles))
+
+true_eq_hosts = partial(eq_hosts, func=lambda x: x)
 
 def test_hosts_decorator_by_itself():
     """
@@ -104,7 +122,6 @@ fake_roles = {
     'r2': ['b', 'c']
 }
 
-@patched_env({'roledefs': fake_roles})
 def test_roles_decorator_by_itself():
     """
     Use of @roles only
@@ -112,27 +129,74 @@ def test_roles_decorator_by_itself():
     @roles('r1')
     def command():
         pass
-    eq_hosts(command, ['a', 'b'])
+    eq_hosts(command, ['a', 'b'], env={'roledefs': fake_roles})
+    eq_effective_roles(command, ['r1'], env={'roledefs': fake_roles})
+
+def test_roles_decorator_overrides_env_roles():
+    """
+    If @roles is used it replaces any env.roles value
+    """
+    @roles('r1')
+    def command():
+        pass
+    eq_effective_roles(command, ['r1'], env={'roledefs': fake_roles,
+                                             'roles': ['r2']})
+
+def test_cli_roles_override_decorator_roles():
+    """
+    If CLI roles are provided they replace roles defined in @roles.
+    """
+    @roles('r1')
+    def command():
+        pass
+    eq_effective_roles(command, ['r2'], cli_roles=['r2'], env={'roledefs': fake_roles})
 
 
-@patched_env({'roledefs': fake_roles})
 def test_hosts_and_roles_together():
     """
     Use of @roles and @hosts together results in union of both
     """
     @roles('r1', 'r2')
+    @hosts('d')
+    def command():
+        pass
+    eq_hosts(command, ['a', 'b', 'c', 'd'], env={'roledefs': fake_roles})
+    eq_effective_roles(command, ['r1', 'r2'], env={'roledefs': fake_roles})
+
+def test_host_role_merge_deduping():
+    """
+    Use of @roles and @hosts dedupes when merging
+    """
+    @roles('r1', 'r2')
     @hosts('a')
     def command():
         pass
-    eq_hosts(command, ['a', 'b', 'c'])
+    # Not ['a', 'a', 'b', 'c'] or etc
+    true_eq_hosts(command, ['a', 'b', 'c'], env={'roledefs': fake_roles})
+
+def test_host_role_merge_deduping_off():
+    """
+    Allow turning deduping off
+    """
+    @roles('r1', 'r2')
+    @hosts('a')
+    def command():
+        pass
+    with settings(dedupe_hosts=False):
+        true_eq_hosts(
+            command,
+            # 'a' 1x host 1x role
+            # 'b' 1x r1 1x r2
+            ['a', 'a', 'b', 'b', 'c'],
+            env={'roledefs': fake_roles}
+        )
+
 
 tuple_roles = {
     'r1': ('a', 'b'),
     'r2': ('b', 'c'),
 }
 
-
-@patched_env({'roledefs': tuple_roles})
 def test_roles_as_tuples():
     """
     Test that a list of roles as a tuple succeeds
@@ -140,17 +204,17 @@ def test_roles_as_tuples():
     @roles('r1')
     def command():
         pass
-    eq_hosts(command, ['a', 'b'])
+    eq_hosts(command, ['a', 'b'], env={'roledefs': tuple_roles})
+    eq_effective_roles(command, ['r1'], env={'roledefs': fake_roles})
 
 
-@patched_env({'hosts': ('foo', 'bar')})
 def test_hosts_as_tuples():
     """
     Test that a list of hosts as a tuple succeeds
     """
     def command():
         pass
-    eq_hosts(command, ['foo', 'bar'])
+    eq_hosts(command, ['foo', 'bar'], env={'hosts': ('foo', 'bar')})
 
 
 def test_hosts_decorator_overrides_env_hosts():
@@ -160,8 +224,7 @@ def test_hosts_decorator_overrides_env_hosts():
     @hosts('bar')
     def command():
         pass
-    eq_hosts(command, ['bar'])
-    assert 'foo' not in get_hosts(command, [], [], [], {'hosts': ['foo']})
+    eq_hosts(command, ['bar'], env={'hosts': ['foo']})
 
 def test_hosts_decorator_overrides_env_hosts_with_task_decorator_first():
     """
@@ -171,16 +234,14 @@ def test_hosts_decorator_overrides_env_hosts_with_task_decorator_first():
     @hosts('bar')
     def command():
         pass
-    eq_hosts(command, ['bar'])
-    assert 'foo' not in get_hosts(command, [], [], {'hosts': ['foo']})
+    eq_hosts(command, ['bar'], env={'hosts': ['foo']})
 
 def test_hosts_decorator_overrides_env_hosts_with_task_decorator_last():
     @hosts('bar')
     @task
     def command():
         pass
-    eq_hosts(command, ['bar'])
-    assert 'foo' not in get_hosts(command, [], [], {'hosts': ['foo']})
+    eq_hosts(command, ['bar'], env={'hosts': ['foo']})
 
 def test_hosts_stripped_env_hosts():
     """
@@ -189,7 +250,7 @@ def test_hosts_stripped_env_hosts():
     def command():
         pass
     myenv = {'hosts': [' foo ', 'bar '], 'roles': [], 'exclude_hosts': []}
-    eq_hosts(command, ['foo', 'bar'], myenv)
+    eq_hosts(command, ['foo', 'bar'], env=myenv)
 
 
 spaced_roles = {
@@ -204,7 +265,22 @@ def test_roles_stripped_env_hosts():
     @roles('r1')
     def command():
         pass
-    eq_hosts(command, ['a', 'b'], {'roledefs': spaced_roles})
+    eq_hosts(command, ['a', 'b'], env={'roledefs': spaced_roles})
+
+
+dict_roles = {
+    'r1': {'hosts': ['a', 'b']},
+    'r2': ['b', 'c'],
+}
+
+def test_hosts_in_role_dict():
+    """
+    Make sure hosts defined in env.roles are cleaned of extra spaces
+    """
+    @roles('r1')
+    def command():
+        pass
+    eq_hosts(command, ['a', 'b'], env={'roledefs': dict_roles})
 
 
 def test_hosts_decorator_expands_single_iterable():
@@ -239,14 +315,17 @@ def test_roles_decorator_expands_single_iterable():
 def dummy(): pass
 
 def test_get_hosts_excludes_cli_exclude_hosts_from_cli_hosts():
-    assert 'foo' not in get_hosts(dummy, ['foo', 'bar'], [], ['foo'])
+    eq_hosts(dummy, ['bar'], cli_hosts=['foo', 'bar'], excluded_hosts=['foo'])
 
 def test_get_hosts_excludes_cli_exclude_hosts_from_decorator_hosts():
-    assert 'foo' not in get_hosts(hosts('foo', 'bar')(dummy), [], [], ['foo'])
+    @hosts('foo', 'bar')
+    def command():
+        pass
+    eq_hosts(command, ['bar'], excluded_hosts=['foo'])
 
-@patched_env({'hosts': ['foo', 'bar'], 'exclude_hosts': ['foo']})
 def test_get_hosts_excludes_global_exclude_hosts_from_global_hosts():
-    assert 'foo' not in get_hosts(dummy, [], [], [])
+    fake_env = {'hosts': ['foo', 'bar'], 'exclude_hosts': ['foo']}
+    eq_hosts(dummy, ['bar'], env=fake_env)
 
 
 
@@ -261,10 +340,15 @@ def test_aborts_on_nonexistent_roles():
     """
     merge([], ['badrole'], [], {})
 
+def test_accepts_non_list_hosts():
+    """
+    Coerces given host string to a one-item list
+    """
+    assert merge('badhosts', [], [], {}) == ['badhosts']
+
 
 lazy_role = {'r1': lambda: ['a', 'b']}
 
-@patched_env({'roledefs': lazy_role})
 def test_lazy_roles():
     """
     Roles may be callables returning lists, as well as regular lists
@@ -272,7 +356,7 @@ def test_lazy_roles():
     @roles('r1')
     def command():
         pass
-    eq_hosts(command, ['a', 'b'])
+    eq_hosts(command, ['a', 'b'], env={'roledefs': lazy_role})
 
 
 #
@@ -342,7 +426,7 @@ class TestTaskAliases(FabricTest):
             ok_("foo_aliased" in funcs)
             ok_("foo_aliased_two" in funcs)
 
-    def test_nested_alias(self):
+    def test_nested_aliases(self):
         f = fabfile("nested_aliases.py")
         with path_prefix(f):
             docs, funcs = load_fabfile(f)
@@ -374,6 +458,16 @@ class TestNamespaces(FabricTest):
             ok_("foo" in funcs)
             ok_("bar" in funcs)
 
+    def test_exception_exclusion(self):
+        """
+        Exception subclasses should not be considered as tasks
+        """
+        exceptions = fabfile("exceptions_fabfile.py")
+        with path_prefix(exceptions):
+            docs, funcs = load_fabfile(exceptions)
+            ok_("some_task" in funcs)
+            ok_("NotATask" not in funcs)
+
     def test_explicit_discovery(self):
         """
         If __all__ is present, only collect the tasks it specifies
@@ -400,12 +494,23 @@ class TestNamespaces(FabricTest):
         Wrapped new-style tasks should preserve their function names
         """
         module = fabfile('decorated_fabfile_with_classbased_task.py')
-        from fabric.state import env
         with path_prefix(module):
             docs, funcs = load_fabfile(module)
             eq_(len(funcs), 1)
             ok_('foo' in funcs)
 
+    def test_class_based_tasks_are_found_with_variable_name(self):
+        """
+        A new-style tasks with undefined name attribute should use the instance
+        variable name.
+        """
+        module = fabfile('classbased_task_fabfile.py')
+        with path_prefix(module):
+            docs, funcs = load_fabfile(module)
+            eq_(len(funcs), 1)
+            ok_('foo' in funcs)
+            eq_(funcs['foo'].name, 'foo')
+
     def test_recursion_steps_into_nontask_modules(self):
         """
         Recursive loading will continue through modules with no tasks
@@ -426,6 +531,16 @@ class TestNamespaces(FabricTest):
             eq_(len(funcs), 1)
             ok_('submodule.classic_task' not in _task_names(funcs))
 
+    def test_task_decorator_plays_well_with_others(self):
+        """
+        @task, when inside @hosts/@roles, should not hide the decorated task.
+        """
+        module = fabfile('decorator_order')
+        with path_prefix(module):
+            docs, funcs = load_fabfile(module)
+            # When broken, crawl() finds None for 'foo' instead.
+            eq_(crawl('foo', funcs), funcs['foo'])
+
 
 #
 # --list output
