@@ -1,28 +1,25 @@
 from __future__ import with_statement
 
-from datetime import datetime
-import copy
-import getpass
 import sys
 
-from nose.tools import with_setup, ok_, raises
-from fudge import (Fake, clear_calls, clear_expectations, patch_object, verify,
-    with_patched_object, patched_context, with_fakes)
+from nose.tools import ok_
+from fudge import (Fake, patch_object, with_patched_object, patched_context,
+                   with_fakes)
 
 from fabric.context_managers import settings, hide, show
 from fabric.network import (HostConnectionCache, join_host_strings, normalize,
-    denormalize, key_filenames, ssh)
-from fabric.io import output_loop
-import fabric.network  # So I can call patch_object correctly. Sigh.
+                            denormalize, key_filenames, ssh, NetworkError)
 from fabric.state import env, output, _get_system_username
 from fabric.operations import run, sudo, prompt
 from fabric.tasks import execute
 from fabric.api import parallel
 from fabric import utils # for patching
 
-from utils import *
-from server import (server, PORT, RESPONSES, PASSWORDS, CLIENT_PRIVKEY, USER,
-    CLIENT_PRIVKEY_PASSPHRASE)
+from mock_streams import mock_streams
+from server import (server, RESPONSES, PASSWORDS, CLIENT_PRIVKEY, USER,
+                    CLIENT_PRIVKEY_PASSPHRASE)
+from utils import (FabricTest, aborts, assert_contains, eq_, password_response,
+                   patched_input, support)
 
 
 #
@@ -546,6 +543,24 @@ class TestConnections(FabricTest):
         with settings(hide('everything'), skip_bad_hosts=True):
             execute(subtask, hosts=['nope.nonexistent.com'])
 
+    @server()
+    def test_host_not_in_known_hosts_exception(self):
+        """
+        Check reject_unknown_hosts exception
+        """
+        with settings(
+            hide('everything'), password=None, reject_unknown_hosts=True,
+            disable_known_hosts=True, abort_on_prompts=True,
+        ):
+            try:
+                run("echo foo")
+            except NetworkError as exc:
+                exp = "Server '[127.0.0.1]:2200' not found in known_hosts"
+                assert str(exc) == exp, "%s != %s" % (exc, exp)
+            else:
+                raise AssertionError("Host connected without valid "
+                                     "fingerprint.")
+
 
 @parallel
 def parallel_subtask():
@@ -613,7 +628,7 @@ class TestSSHConfig(FabricTest):
         """
         Global Port should NOT override nondefault env.port
         """
-        with settings(port="777"):
+        with settings(port="777", use_ssh_config=False):
             eq_(normalize("localhost")[2], "777")
 
     def test_specific_port_with_default_env(self):
