@@ -1,14 +1,13 @@
 from __future__ import with_statement
 
 import sys
-import traceback
 from unittest import TestCase
 
 from fudge import Fake, patched_context, with_fakes
 from fudge.patcher import with_patched_object
 from nose.tools import eq_, raises
 
-from fabric.state import output, env
+from fabric.state import output
 from fabric.utils import warn, indent, abort, puts, fastprint, error, RingBuffer
 from fabric import utils  # For patching
 from fabric.api import local, quiet
@@ -29,19 +28,19 @@ def test_warn():
 
 
 def test_indent():
-    for description, input, output in (
+    for description, input_, output_ in (
         ("Sanity check: 1 line string",
             'Test', '    Test'),
         ("List of strings turns in to strings joined by \\n",
             ["Test", "Test"], '    Test\n    Test'),
     ):
         eq_.description = "indent(): %s" % description
-        yield eq_, indent(input), output
+        yield eq_, indent(input_), output_
         del eq_.description
 
 
 def test_indent_with_strip():
-    for description, input, output in (
+    for description, input_, output_ in (
         ("Sanity check: 1 line string",
             indent('Test', strip=True), '    Test'),
         ("Check list of strings",
@@ -51,7 +50,7 @@ def test_indent_with_strip():
             '    Test\n    Test'),
     ):
         eq_.description = "indent(strip=True): %s" % description
-        yield eq_, input, output
+        yield eq_, input_, output_
         del eq_.description
 
 
@@ -172,7 +171,6 @@ def test_puts_without_prefix():
     puts() shouldn't prefix output with env.host_string if show_prefix is False
     """
     s = "my output"
-    h = "localhost"
     puts(s, show_prefix=False)
     eq_(sys.stdout.getvalue(), "%s" % (s + "\n"))
 
@@ -323,3 +321,31 @@ class TestRingBuffer(TestCase):
         self.b.extend("abcde")
         self.b.extend("fgh")
         eq_(self.b, ['d', 'e', 'f', 'g', 'h'])
+
+    def test_plus_equals(self):
+        self.b += "abcdefgh"
+        eq_(self.b, ['d', 'e', 'f', 'g', 'h'])
+
+    def test_oversized_extend(self):
+        self.b.extend("abcdefghijklmn")
+        eq_(self.b, ['j', 'k', 'l', 'm', 'n'])
+
+    def test_zero_maxlen_append(self):
+        b = RingBuffer([], maxlen=0)
+        b.append('a')
+        eq_(b, [])
+
+    def test_zero_maxlen_extend(self):
+        b = RingBuffer([], maxlen=0)
+        b.extend('abcdefghijklmnop')
+        eq_(b, [])
+
+    def test_None_maxlen_append(self):
+        b = RingBuffer([], maxlen=None)
+        b.append('a')
+        eq_(b, ['a'])
+
+    def test_None_maxlen_extend(self):
+        b = RingBuffer([], maxlen=None)
+        b.extend('abcdefghijklmnop')
+        eq_(''.join(b), 'abcdefghijklmnop')
